///////////////////////////////////////////////////////////////////////////
//            Hyper Juke, Audio-CD database with media player.           //
//                Copyright (C) 1990,1996 Fumio Kawamata                 //
//                                                                       //
//  This program is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU General Public License as published by //
//  the Free Software Foundation; version 2 of the License.              //                                                    //
//                                                                       //
//  This program is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General    //
//  Public License in the directory "GNUGPL" for more details.           //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit Player;

interface

uses
  MPlayer, MyFunc,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DB, DBTables, Grids, DBGrids;

type
  TformPlayer = class(TForm)
    timer1: TTimer;
    panelBottom: TPanel;
    bitbtnClose: TBitBtn;
    panelPlayerButtons: TPanel;
    bitbtnPause: TBitBtn;
    bitbtnStop: TBitBtn;
    bitbtnEject: TBitBtn;
    bitbtnTrackBwd: TBitBtn;
    panelTop: TPanel;
    panelStatus: TPanel;
    labelTimeValue: TLabel;
    labelTrackValue: TLabel;
    labelTrack: TLabel;
    labelTime: TLabel;
    tableProg: TTable;
    datasourceProg: TDataSource;
    dbgridProg: TDBGrid;
    tableProgUNIT_ID: TStringField;
    tableProgSLOT_ID: TStringField;
    tableProgDISC_ID: TStringField;
    bitbtnPlay: TBitBtn;
    bitbtnTrackFwd: TBitBtn;
    tableMusic: TTable;
    tableProgWORK_TITLE: TStringField;
    tableProgWORK_PLAYER: TStringField;
    editStatus: TEdit;
    mediaplayerCD: TMediaPlayer;
    tableProgNO: TSmallintField;
    tableProgTRACK1: TSmallintField;
    tableProgTRACK2: TSmallintField;
    bitbtnDetailsOfMusic: TBitBtn;
    procedure bitbtnPlayClick(Sender: TObject);
    procedure bitbtnPauseClick(Sender: TObject);
    procedure bitbtnStopClick(Sender: TObject);
    procedure bitbtnTrackFwdClick(Sender: TObject);
    procedure bitbtnTrackBwdClick(Sender: TObject);
    procedure bitbtnEjectClick(Sender: TObject);
    procedure timer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bitbtnCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure mediaplayerCDNotify(Sender: TObject);
    procedure tableProgCalcFields(DataSet: TDataset);
    procedure dbgridProgColExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure dbgridProgDrawDataCell(Sender: TObject; const Rect: TRect;
      Field: TField; State: TGridDrawState);
  private
    procedure OpenCdAudio;
    procedure PlayCD;
    procedure PlayNext;
    procedure PlayNextAuto;
  public
  end;

var
  formPlayer: TformPlayer;

implementation

{$R *.DFM}
type
  MSFRec = record
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
    NotUsed : byte;
  end;
  TMSFRec = record
    Track   : byte;
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
  end;
var
  CdAudioIsOpened    : Boolean;
  EndPosTMSF         : Longint;
  Paused             : Boolean;
const
  CRLF    = chr($0D)+chr($0A);


//*************************************************************************
//* Opens CdAudio device
//*************************************************************************
procedure TformPlayer.OpenCdAudio;
const
  CRLF = chr($0D)+chr($0D);
var
  MessageStr     : string;
begin
  CdAudioIsOpened:=True;
  try
    mediaplayerCD.Open;
  except
    MessageStr:='Could not open the CD-Audio player.  '+CRLF+
                'It may be occupied by another process.';

    MessageDlg(MessageStr, mtWarning, [mbOK], 0);
    mediaplayerCD.Close;
    CdAudioIsOpened:=False;
  end;
end;


//*************************************************************************
//* Plays next track of the CdAudio player
//*************************************************************************
procedure TformPlayer.PlayNext;
begin
  tableProg.Next;
  if not(tableProg.Eof) then begin
    PlayCD;
  end;
end;


//*************************************************************************
//* Plays next track of the CdAudio automatically
//*************************************************************************
procedure TformPlayer.PlayNextAuto;
begin
  tableProg.Next;
  if not(tableProg.Eof) then begin
    PlayCD;
  end
  else begin
    mediaplayerCD.Rewind;
  end;
end;


//*************************************************************************
//* Plays CdAudio player
//*************************************************************************
procedure TformPlayer.PlayCD;
var
  MSF            : Longint;
  StartPosTMSF   : Longint;
  T1             : byte;
  T2             : byte;

begin
  T1:=tableProg.FieldByName('Track1').AsInteger;
  T2:=tableProg.FieldByName('Track2').AsInteger;
  StartPosTMSF:=T1;
  mediaplayerCD.TimeFormat:=tfMSF;
  MSF:=mediaplayerCD.TrackLength[T2];
  TMSFRec(EndPosTMSF).Track:=T2;
  TMSFRec(EndPosTMSF).Minutes:=MSFRec(MSF).Minutes;
  TMSFRec(EndPosTMSF).Seconds:=MSFRec(MSF).Seconds;
  TMSFRec(EndPosTMSF).Frames:=MSFRec(MSF).Frames;
  with TMSFRec(EndPosTMSF) do begin
    editStatus.Text:='Check at Track '+IntToStr(Track)+', Time '+
    IntToStr(Minutes)+':'+IntToStr(Seconds);
  end;
  with mediaplayerCD do begin
    TimeFormat:=tfTMSF;
    StartPos:=StartPosTMSF;
    EndPos:=EndPosTMSF;
    Notify:=True;
    Play;
  end;
  Paused:=False;
  bitbtnPlay.Enabled:=False;
  bitbtnPause.Enabled:=True;
  bitbtnStop.Enabled:=True;
  bitbtnTrackFwd.Enabled:=True;
  bitbtnTrackBwd.Enabled:=True;
  bitbtnEject.Enabled:=False;
end;


//*************************************************************************
//* Plays CdAudio player
//*************************************************************************
procedure TformPlayer.bitbtnPlayClick(Sender: TObject);
begin
  PlayCD;
end;


//*************************************************************************
//* Pauses CdAudio player
//*************************************************************************
procedure TformPlayer.bitbtnPauseClick(Sender: TObject);
begin
  mediaplayerCD.Pause;
  if Paused=True then begin
    mediaplayerCD.EndPos:=EndPosTMSF;
    mediaplayerCD.Play;
    Paused:=False;
    bitbtnPlay.Enabled:=False;
    bitbtnPause.Enabled:=True;
    bitbtnStop.Enabled:=True;
    bitbtnTrackFwd.Enabled:=True;
    bitbtnTrackBwd.Enabled:=True;
    bitbtnEject.Enabled:=False;
  end
  else begin
    Paused:=True;
    bitbtnPlay.Enabled:=False;
    bitbtnPause.Enabled:=True;
    bitbtnStop.Enabled:=False;
    bitbtnTrackFwd.Enabled:=False;
    bitbtnTrackBwd.Enabled:=False;
    bitbtnEject.Enabled:=False;
  end;
end;


//*************************************************************************
//* Stops CdAudio player
//*************************************************************************
procedure TformPlayer.bitbtnStopClick(Sender: TObject);
begin
  mediaplayerCD.Stop;
  mediaplayerCD.Rewind;
  Paused:=False;
  bitbtnPlay.Enabled:=True;
  bitbtnPause.Enabled:=False;
  bitbtnStop.Enabled:=False;
  bitbtnTrackFwd.Enabled:=False;
  bitbtnTrackBwd.Enabled:=False;
  bitbtnEject.Enabled:=True;
end;


//*************************************************************************
//* Plays next program
//*************************************************************************
procedure TformPlayer.bitbtnTrackFwdClick(Sender: TObject);
begin
   PlayNext;
end;


//*************************************************************************
//* Playes previous program
//*************************************************************************
procedure TformPlayer.bitbtnTrackBwdClick(Sender: TObject);
begin
  tableProg.Prior;
  if not(tableProg.Bof) then begin
    PlayCD;
  end;
end;


//*************************************************************************
//* Ejects CdAudio player
//*************************************************************************
procedure TformPlayer.bitbtnEjectClick(Sender: TObject);
begin
  mediaplayerCD.Eject;
  mediaplayerCD.Rewind;
  Paused:=False;
end;


//*************************************************************************
//* Executes interval timer
//*************************************************************************
procedure TformPlayer.timer1Timer(Sender: TObject);
const
  ModeStr   : array[TMPModes] of string[10]
              =('Not Ready','Stopped','Playing','Recording','Seeking',
                'Paused', 'Open');
var
  TimeStr   : String;
  TrackStr  : String;
  TMSF      : Longint;
begin
  if CdAudioIsOpened=False then Exit;
  if mediaplayerCD.Mode in [mpNotReady, mpOpen] then begin
    TrackStr:='--';
    TimeStr:='BUSY';
  end
  else begin
    TMSF:=mediaplayerCD.Position;
    TrackStr:=TMSFtoTrack(TMSF);
    TimeStr:=copy(TMSFtoTime(TMSF),1,5);
  end;
    labelTrackValue.Caption:=TrackStr;
    labelTimeValue.Caption:=TimeStr;
end;


//*************************************************************************
//* Executes when form close event occurs
//*************************************************************************
procedure TformPlayer.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  mediaplayerCD.Close;
  timer1.Enabled:=False;
end;


//*************************************************************************
//* Close myself
//*************************************************************************
procedure TformPlayer.bitbtnCloseClick(Sender: TObject);
begin
  Close;
end;


//*************************************************************************
//* Executes when form show event occurs
//*************************************************************************
procedure TformPlayer.FormShow(Sender: TObject);
begin
  OpenCdAudio;
  timer1.Enabled:=True;
end;


//*************************************************************************
//* Executes when CDNotify event occurs
//*************************************************************************
procedure TformPlayer.mediaplayerCDNotify(Sender: TObject);
var
  ResultStr  : string;
begin
  case mediaplayerCD.NotifyValue of
    nvSuccessful : ResultStr:='Success';
    nvSuperseded : ResultStr:='Superseded';
    nvAborted    : ResultStr:='Aborted';
    nvFailure    : ResultStr:='Filure';
    else ResultStr:='UnKnown';
  end;
  if mediaplayerCD.NotifyValue=nvSuccessful then begin
    PlayNext;
  end;
end;


//*************************************************************************
//* Sets items into TITLE and PLAYER fields
//*************************************************************************
procedure TformPlayer.tableProgCalcFields(DataSet: TDataset);
var
  Key1 : String;
  Key2 : Integer;
  Key3 : Integer;
begin
  Key1:=DataSet.FieldByName('DISC_ID').AsString;
  Key2:=DataSet.FieldByName('TRACK1').AsInteger;
  Key3:=DataSet.FieldByName('TRACK2').AsInteger;
  If tableMusic.FindKey([Key1, Key2, Key3]) then begin
    tableProgWORK_TITLE.AsString :=tableMusic.FieldByName('TITLE').AsString;
    tableProgWORK_PLAYER.AsString:=tableMusic.FieldByName('PLAYER').AsString;
  end;
end;


//******************************************************************************
//* Invalidates the dbgridProg object
//******************************************************************************
procedure TformPlayer.dbgridProgColExit(Sender: TObject);
begin
  dbgridProg.Invalidate;
end;


//******************************************************************************
//* Sets the directory of the playing program database
//******************************************************************************
procedure TformPlayer.FormCreate(Sender: TObject);
begin
  tableProg.Active:=False;
  tableProg.Databasename:=ExtractFilePath(ParamStr(0))+'WORK';
  tableProg.Active:=True;
  tableProg.Refresh;
end;


//******************************************************************************
//* Colors selected record of the dbgrid
//******************************************************************************
procedure TformPlayer.dbgridProgDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
begin
  if gdSelected in State then begin
    dbgridProg.Canvas.Brush.Color:=clAqua;
    dbgridProg.Canvas.Font.Color:=clBlack;
    dbgridProg.Canvas.Font.Style:=[fsBold];
  end;
  dbgridProg.DefaultDrawDataCell(Rect, Field, State);
end;


end.
